@@ -29,7 +29,7 @@ NCOLOR = False
 def _init_model_list(parent):
     models.MODEL_DIR.mkdir(parents=True, exist_ok=True)
     parent.model_list_path = os.fspath(models.MODEL_DIR.joinpath('gui_models.txt'))
-    parent.model_strings = models.MODEL_NAMES.copy()
+    parent.model_strings = []
     if not os.path.exists(parent.model_list_path):
         textfile = open(parent.model_list_path, 'w')
         textfile.close()
@@ -38,50 +38,60 @@ def _init_model_list(parent):
             lines = [line.rstrip() for line in textfile]
             if len(lines) > 0:
                 parent.model_strings.extend(lines)
-    parent.permanent_model = [True for i in range(len(parent.model_strings))]
-
-def _add_model(parent, filename=None, permanent=True):
+    
+def _add_model(parent, filename=None, load_model=True):
     if filename is None:
         name = QFileDialog.getOpenFileName(
             parent, "Add model to GUI"
             )
         filename = name[0]
     fname = os.path.split(filename)[-1]
-    shutil.copyfile(filename, os.fspath(models.MODEL_DIR.joinpath(fname)))
+    try:
+        shutil.copyfile(filename, os.fspath(models.MODEL_DIR.joinpath(fname)))
+    except shutil.SameFileError:
+        pass
     print(f'GUI_INFO: {filename} copied to models folder {os.fspath(models.MODEL_DIR)}')
     with open(parent.model_list_path, 'a') as textfile:
         textfile.write(fname + '\n')
     parent.ModelChoose.addItems([fname])
     parent.model_strings.append(fname)
-    parent.permanent_model.append(permanent)
-    parent.ModelChoose.setCurrentIndex(len(parent.model_strings) - 1)
-    parent.NetAvg.setCurrentIndex(1)
+    if len(parent.model_strings) > 0:
+        parent.ModelButton.setStyleSheet(parent.styleUnpressed)
+        parent.ModelButton.setEnabled(True)
+    
+    for ind, model_string in enumerate(parent.model_strings[:-1]):
+        if model_string == fname:
+            _remove_model(parent, ind=ind+1, verbose=False)
 
-#def _remove_non_permanent_models(parent):
-#    for perm in parent.permanent_model:
-#        if not perm: 
+    parent.ModelChoose.setCurrentIndex(len(parent.model_strings))
+    if load_model:
+        parent.model_choose(len(parent.model_strings))
 
-def _remove_model(parent, ind=None):
+def _remove_model(parent, ind=None, verbose=True):
     if ind is None:
         ind = parent.ModelChoose.currentIndex()
-    if ind > len(models.MODEL_NAMES)-1:
-        print(f'GUI_INFO: deleting {parent.model_strings[ind]} from GUI')
-        parent.ModelChoose.removeItem(ind)
+    if ind > 0:
+        ind -= 1
+        if verbose:
+            print(f'GUI_INFO: deleting {parent.model_strings[ind]} from GUI')
+        parent.ModelChoose.removeItem(ind+1)
         del parent.model_strings[ind]
-        del parent.permanent_model[ind]
-        custom_strings = parent.model_strings[len(models.MODEL_NAMES):]
+        custom_strings = parent.model_strings
         if len(custom_strings) > 0:
             with open(parent.model_list_path, 'w') as textfile:
                 for fname in custom_strings:
                     textfile.write(fname + '\n')
-            parent.ModelChoose.setCurrentIndex(len(parent.model_strings) - 1)
+            parent.ModelChoose.setCurrentIndex(len(parent.model_strings))
         else:
             # write empty file
             textfile = open(parent.model_list_path, 'w')
             textfile.close()
             parent.ModelChoose.setCurrentIndex(0)
+            parent.ModelButton.setStyleSheet(parent.styleInactive)
+            parent.ModelButton.setEnabled(False)
     else:
-        print('ERROR: cannot remove built-in model, select custom model to delete')
+        print('ERROR: no model selected to delete')
+
     
 
 def _get_train_set(image_names):
@@ -92,21 +102,16 @@ def _get_train_set(image_names):
         label_name = None
         if os.path.exists(image_name + '_seg.npy'):
             dat = np.load(image_name + '_seg.npy', allow_pickle=True).item()
-            masks = dat['masks']
-            imsave(image_name + '_masks.tif', masks)
-            label_name = image_name + '_masks.tif'
-        else:
-            mask_filter = '_masks'
-            if os.path.exists(image_name + mask_filter + '.tif'):
-                label_name = image_name + mask_filter + '.tif'
-            elif os.path.exists(image_name + mask_filter + '.tiff'):
-                label_name = image_name + mask_filter + '.tiff'
-            elif os.path.exists(image_name + mask_filter + '.png'):
-                label_name = image_name + mask_filter + '.png'
+            masks = dat['masks'].squeeze()
+            if masks.ndim==2:
+                fastremap.renumber(masks, in_place=True)
+                label_name = image_name + '_seg.npy'
+            else:
+                print(f'GUI_INFO: _seg.npy found for {image_name} but masks.ndim!=2')
         if label_name is not None:
             train_files.append(image_name_full)
             train_data.append(imread(image_name_full))
-            train_labels.append(imread(label_name))
+            train_labels.append(masks)
     return train_data, train_labels, train_files
 
 def _load_image(parent, filename=None, load_seg=True):
@@ -128,12 +133,15 @@ def _load_image(parent, filename=None, load_seg=True):
             return
         elif parent.autoloadMasks.isChecked():
             mask_file = os.path.splitext(filename)[0]+'_masks'+os.path.splitext(filename)[-1]
+            mask_file = os.path.splitext(filename)[0]+'_masks.tif' if not os.path.isfile(mask_file) else mask_file
             load_mask = True if os.path.isfile(mask_file) else False
     try:
+        print(f'GUI_INFO: loading image: {filename}')
         image = imread(filename)
         parent.loaded = True
-    except:
+    except Exception as e:
         print('ERROR: images not compatible')
+        print(f'ERROR: {e}')
 
     if parent.loaded:
         parent.reset()
@@ -145,8 +153,6 @@ def _load_image(parent, filename=None, load_seg=True):
         parent.enable_buttons()
         if load_mask:
             _load_masks(parent, filename=mask_file)
-        parent.threshslider.setEnabled(False)
-        parent.probslider.setEnabled(False)
             
 
 
@@ -208,11 +214,19 @@ def _initialize_images(parent, image, resize, X2):
     if parent.stack.ndim < 4:
         parent.onechan=True
         parent.stack = parent.stack[:,:,:,np.newaxis]
+    
     parent.imask=0
     parent.Ly, parent.Lx = parent.stack.shape[1:3]
-    parent.layers = 0*np.ones((parent.NZ,parent.Ly,parent.Lx,4), np.uint8)
-    if parent.autobtn.isChecked() or len(parent.saturation)!=parent.NZ:
+    parent.layerz = 255 * np.ones((parent.Ly,parent.Lx,4), 'uint8')
+    print(parent.layerz.shape)
+    if parent.autobtn.isChecked():
         parent.compute_saturation()
+    elif len(parent.saturation) != parent.NZ:
+        parent.saturation = []
+        for n in range(parent.NZ):
+            parent.saturation.append([0, 255])
+        parent.slider.setLow(0)
+        parent.slider.setHigh(255)
     parent.compute_scale()
     parent.currentZ = int(np.floor(parent.NZ/2))
     parent.scroll.setValue(parent.currentZ)
@@ -293,7 +307,7 @@ def _load_seg(parent, filename=None, image=None, image_file=None):
                     color = parent.colormap[col_rand,:3]
                 median = parent.add_mask(points=outline, color=color)
                 if median is not None:
-                    parent.cellcolors.append(color)
+                    parent.cellcolors = np.append(parent.cellcolors, color[np.newaxis,:], axis=0)
                     parent.ncells+=1
         else:
             if dat['masks'].ndim==2:
@@ -303,21 +317,23 @@ def _load_seg(parent, filename=None, image=None, image_file=None):
                 dat['masks'] += 1
                 dat['outlines'] += 1
             parent.ncells = dat['masks'].max()
-            if 'colors' in dat:
+            if 'colors' in dat and len(dat['colors'])==dat['masks'].max():
                 colors = dat['colors']
             else:
                 colors = parent.colormap[:parent.ncells,:3]
             parent.cellpix = dat['masks']
             parent.outpix = dat['outlines']
-            parent.cellcolors.extend(colors)
-            parent.draw_masks()
+            parent.cellcolors = np.append(parent.cellcolors, colors, axis=0)
+            
+            parent.draw_layer()
             if 'est_diam' in dat:
                 parent.Diameter.setText('%0.1f'%dat['est_diam'])
                 parent.diameter = dat['est_diam']
                 parent.compute_scale()
 
-            if parent.masksOn or parent.outlinesOn and not (parent.masksOn and parent.outlinesOn):
-                parent.redraw_masks(masks=parent.masksOn, outlines=parent.outlinesOn)
+        if 'manual_changes' in dat: 
+            parent.track_changes = dat['manual_changes']
+            print('GUI_INFO: loaded in previous changes')    
         if 'zdraw' in dat:
             parent.zdraw = dat['zdraw']
         else:
@@ -344,21 +360,20 @@ def _load_seg(parent, filename=None, image=None, image_file=None):
                 parent.flows[0] = cv2.resize(parent.flows[0].squeeze(), (Lx, Ly), interpolation=cv2.INTER_NEAREST)[np.newaxis,...]
                 parent.flows[1] = cv2.resize(parent.flows[1].squeeze(), (Lx, Ly), interpolation=cv2.INTER_NEAREST)[np.newaxis,...]
             if parent.NZ==1:
-                parent.threshslider.setEnabled(True)
-                parent.probslider.setEnabled(True)
+                parent.recompute_masks = True
             else:
-                parent.threshslider.setEnabled(False)
-                parent.probslider.setEnabled(False)
+                parent.recompute_masks = False
+                
         except:
             try:
                 if len(parent.flows[0])>0:
                     parent.flows = parent.flows[0]
             except:
                 parent.flows = [[],[],[],[],[[]]]
-            parent.threshslider.setEnabled(False)
-            parent.probslider.setEnabled(False)
-            
+            parent.recompute_masks = False
+
     parent.enable_buttons()
+    parent.update_layer()
     del dat
     gc.collect()
 
@@ -369,6 +384,7 @@ def _load_masks(parent, filename=None):
             parent, "Load masks (PNG or TIFF)"
             )
         filename = name[0]
+    print(f'GUI_INFO: loading masks: {filename}')
     masks = imread(filename)
     outlines = None
     if masks.ndim>3:
@@ -389,21 +405,25 @@ def _load_masks(parent, filename=None):
     if masks.shape[0]!=parent.NZ:
         print('ERROR: masks are not same depth (number of planes) as image stack')
         return
-    print(f'GUI_INFO: {len(np.unique(masks))-1} masks found in {filename}')
 
     _masks_to_gui(parent, masks, outlines)
-
+    del masks 
+    gc.collect()
+    parent.update_layer()
     parent.update_plot()
 
 def _masks_to_gui(parent, masks, outlines=None):
     """ masks loaded into GUI """
     # get unique values
     shape = masks.shape
-    
+    masks = masks.flatten()
     fastremap.renumber(masks, in_place=True)
-    masks = np.reshape(masks, shape)
+    masks = masks.reshape(shape)
     masks = masks.astype(np.uint16) if masks.max()<2**16-1 else masks.astype(np.uint32)
     parent.cellpix = masks
+    if parent.cellpix.ndim == 2:
+        parent.cellpix = parent.cellpix[np.newaxis,:,:]
+    print(f'GUI_INFO: {masks.max()} masks found')
 
     # get outlines
     if outlines is None: # parent.outlinesOn
@@ -421,14 +441,14 @@ def _masks_to_gui(parent, masks, outlines=None):
 
     parent.ncells = parent.cellpix.max()
     colors = parent.colormap[:parent.ncells, :3]
-
-    parent.cellcolors = list(np.concatenate((np.array([[255,255,255]]), colors), axis=0).astype(np.uint8))
-    parent.draw_masks()
-    parent.redraw_masks(masks=parent.masksOn, outlines=parent.outlinesOn) # add to obey outline/mask setting upon recomputing 
+    print('GUI_INFO: creating cellcolors and drawing masks')
+    parent.cellcolors = np.concatenate((np.array([[255,255,255]]), colors), axis=0).astype(np.uint8)
+    parent.draw_layer()
     if parent.ncells>0:
         parent.toggle_mask_ops()
     parent.ismanual = np.zeros(parent.ncells, bool)
     parent.zdraw = list(-1*np.ones(parent.ncells, np.int16))
+    parent.update_layer()
     parent.update_plot()
 
 def _save_png(parent):
@@ -457,6 +477,7 @@ def _save_sets(parent):
     """ save masks to *_seg.npy """
     filename = parent.filename
     base = os.path.splitext(filename)[0]
+    flow_threshold, cellprob_threshold = parent.get_thresholds()
     if parent.NZ > 1 and parent.is_stack:
         np.save(base + '_seg.npy',
                 {'outlines': parent.outpix,
@@ -465,7 +486,11 @@ def _save_sets(parent):
                  'current_channel': (parent.color-2)%5,
                  'filename': parent.filename,
                  'flows': parent.flows,
-                 'zdraw': parent.zdraw})
+                 'zdraw': parent.zdraw,
+                 'model_path': parent.current_model_path if hasattr(parent, 'current_model_path') else 0,
+                 'flow_threshold': flow_threshold,
+                 'cellprob_threshold': cellprob_threshold
+                 })
     else:
         image = parent.chanchoose(parent.stack[parent.currentZ].copy())
         if image.ndim < 4:
@@ -481,6 +506,8 @@ def _save_sets(parent):
                  'flows': parent.flows,
                  'ismanual': parent.ismanual,
                  'manual_changes': parent.track_changes,
-                 'model_path': parent.current_model_path if hasattr(parent, 'current_model_path') else 0})
+                 'model_path': parent.current_model_path if hasattr(parent, 'current_model_path') else 0,
+                 'flow_threshold': flow_threshold,
+                 'cellprob_threshold': cellprob_threshold})
     #print(parent.point_sets)
     print('GUI_INFO: %d ROIs saved to %s'%(parent.ncells, base + '_seg.npy'))
