@@ -6,11 +6,6 @@ import logging
 transforms_logger = logging.getLogger(__name__)
 
 from . import dynamics, utils
-try:
-    import omnipose, edt, fastremap
-    OMNI_INSTALLED = True
-except:
-    OMNI_INSTALLED = False
 
 def _taper_mask(ly=224, lx=224, sig=7.5):
     bsize = max(224, max(ly, lx))
@@ -189,20 +184,14 @@ def make_tiles(imgi, bsize=224, augment=False, tile_overlap=0.1):
         
     return IMG, ysub, xsub, Ly, Lx
 
-# needs to have a wider range to avoid weird effects with few cells in frame
-# also turns out previous formulation can give negative numbers, messes up log operations etc. 
-def normalize99(Y,lower=0.01,upper=99.99,omni=False):
-    """ normalize image so 0.0 is 0.01st percentile and 1.0 is 99.99th percentile """
-    if omni and OMNI_INSTALLED:
-        X = omnipose.utils.normalize99(Y)
-    else:
-        X = Y.copy()
-        x01 = np.percentile(X, 1)
-        x99 = np.percentile(X, 99)
-        X = (X - x01) / (x99 - x01)
+def normalize99(Y, lower=1,upper=99):
+    """ normalize image so 0.0 is 1st percentile and 1.0 is 99th percentile """
+    X = Y.copy()
+    x01 = np.percentile(X, lower)
+    x99 = np.percentile(X, upper)
+    X = (X - x01) / (x99 - x01)
     return X
 
-
 def move_axis(img, m_axis=-1, first=True):
     """ move axis m_axis to first or last position """
     if m_axis==-1:
@@ -249,7 +238,7 @@ def update_axis(m_axis, to_squeeze, ndim):
 
 def convert_image(x, channels, channel_axis=None, z_axis=None,
                   do_3D=False, normalize=True, invert=False,
-                  nchan=2, omni=False):
+                  nchan=2):
     """ return image with z first, channels last and normalized intensities """
         
     # squeeze image, and if channel_axis or z_axis given, transpose image
@@ -312,7 +301,7 @@ def convert_image(x, channels, channel_axis=None, z_axis=None,
                                 axis=-1)
             
     if normalize or invert:
-        x = normalize_img(x, invert=invert, omni=omni)
+        x = normalize_img(x, invert=invert)
         
     return x
 
@@ -372,7 +361,7 @@ def reshape(data, channels=[0,0], chan_first=False):
             data = np.transpose(data, (2,0,1))
     return data
 
-def normalize_img(img, axis=-1, invert=False, omni=False):
+def normalize_img(img, axis=-1, invert=False):
     """ normalize each channel of the image so that so that 0.0=1st percentile
     and 1.0=99th percentile of image intensities
 
@@ -385,6 +374,8 @@ def normalize_img(img, axis=-1, invert=False, omni=False):
 
     axis: channel axis to loop over for normalization
 
+    invert: invert image (useful if cells are dark instead of bright)
+
     Returns
     ---------------
 
@@ -404,7 +395,7 @@ def normalize_img(img, axis=-1, invert=False, omni=False):
         i99 = np.percentile(img[k],99)
         i1 = np.percentile(img[k],1)
         if i99 - i1 > +1e-3: #np.ptp(img[k]) > 1e-3:
-            img[k] = normalize99(img[k],omni=omni)
+            img[k] = normalize99(img[k])
             if invert:
                 img[k] = -1*img[k] + 1   
         else:
@@ -412,7 +403,7 @@ def normalize_img(img, axis=-1, invert=False, omni=False):
     img = np.moveaxis(img, 0, axis)
     return img
 
-def reshape_train_test(train_data, train_labels, test_data, test_labels, channels, normalize=True, omni=False):
+def reshape_train_test(train_data, train_labels, test_data, test_labels, channels, normalize=True):
     """ check sizes and reshape train and test data for training """
     nimg = len(train_data)
     # check that arrays are correct size
@@ -440,7 +431,7 @@ def reshape_train_test(train_data, train_labels, test_data, test_labels, channel
 
     # make data correct shape and normalize it so that 0 and 1 are 1st and 99th percentile of data
     train_data, test_data, run_test = reshape_and_normalize_data(train_data, test_data=test_data, 
-                                                                 channels=channels, normalize=normalize, omni=omni)
+                                                                 channels=channels, normalize=normalize)
 
     if train_data is None:
         error_message = 'training data do not all have the same number of channels'
@@ -453,7 +444,7 @@ def reshape_train_test(train_data, train_labels, test_data, test_labels, channel
 
     return train_data, train_labels, test_data, test_labels, run_test
 
-def reshape_and_normalize_data(train_data, test_data=None, channels=None, normalize=True, omni=False):
+def reshape_and_normalize_data(train_data, test_data=None, channels=None, normalize=True):
     """ inputs converted to correct shapes for *training* and rescaled so that 0.0=1st percentile
     and 1.0=99th percentile of image intensities in each channel
 
@@ -496,13 +487,14 @@ def reshape_and_normalize_data(train_data, test_data=None, channels=None, normal
             return train_data, test_data, run_test
         nimg = len(data)
         for i in range(nimg):
-            data[i] = move_min_dim(data[i], force=True)
             if channels is not None:
+                data[i] = move_min_dim(data[i], force=True)
                 data[i] = reshape(data[i], channels=channels, chan_first=True)
             if data[i].ndim < 3:
                 data[i] = data[i][np.newaxis,:,:]
             if normalize:
-                data[i] = normalize_img(data[i], axis=0, omni=omni)
+                data[i] = normalize_img(data[i], axis=0)
+     
         nchan = [data[i].shape[0] for i in range(nimg)]
     run_test = True
     return train_data, test_data, run_test
@@ -604,82 +596,10 @@ def pad_image_ND(img0, div=16, extra = 1):
     xsub = np.arange(ypad1, ypad1+Lx)
     return I, ysub, xsub
 
-
-def random_rotate_and_resize(X, Y=None, scale_range=1., gamma_range=0.5, xy = (224,224), 
-                             do_flip=True, rescale=None, unet=False,
-                             inds=None, depth=0, omni=False):
-    """ augmentation by random rotation and resizing
-
-        X and Y are lists or arrays of length nimg, with dims channels x Ly x Lx (channels optional)
-
-        Parameters
-        ----------
-        X: LIST of ND-arrays, float
-            list of image arrays of size [nchan x Ly x Lx] or [Ly x Lx]
-
-        Y: LIST of ND-arrays, float (optional, default None)
-            list of image labels of size [nlabels x Ly x Lx] or [Ly x Lx]. The 1st channel
-            of Y is always nearest-neighbor interpolated (assumed to be masks or 0-1 representation).
-            If Y.shape[0]==3 and not unet, then the labels are assumed to be [cell probability, Y flow, X flow]. 
-            If unet, second channel is dist_to_bound.
-
-        scale_range: float (optional, default 1.0)
-            Range of resizing of images for augmentation. Images are resized by
-            (1-scale_range/2) + scale_range * np.random.rand()
-        
-        gamma_range: float (optional, default 0.5)
-           Images are gamma-adjusted im**gamma for gamma in (1-gamma_range,1+gamma_range) 
-
-        xy: tuple, int (optional, default (224,224))
-            size of transformed images to return
-
-        do_flip: bool (optional, default True)
-            whether or not to flip images horizontally
-
-        rescale: array, float (optional, default None)
-            how much to resize images by before performing augmentations
-
-        unet: bool (optional, default False)
-
-        Returns
-        -------
-        imgi: ND-array, float
-            transformed images in array [nimg x nchan x xy[0] x xy[1]]
-
-        lbl: ND-array, float
-            transformed labels in array [nimg x nchan x xy[0] x xy[1]]
-
-        scale: array, float
-            amount each image was resized by
-
-    """
-    scale_range = max(0, min(2, float(scale_range))) # limit overall range to [0,2] i.e. 1+-1 
-    
-    if inds is None: # only relevant when debugging 
-        nimg = len(X)
-        inds = np.arange(nimg)
-    
-    if omni and OMNI_INSTALLED:
-        return omnipose.core.random_rotate_and_resize(X, Y=Y, scale_range=scale_range, gamma_range=gamma_range, 
-                                                 xy=xy, do_flip=do_flip, rescale=rescale, inds=inds)
-    else:
-        # backwards compatibility; completely 'stock', no gamma augmentation or any other extra frills. 
-        # [Y[i][1:] for i in inds] is necessary because the original transform function does not use masks (entry 0). 
-        # This used to be done in the original function call. 
-        return original_random_rotate_and_resize(X, Y=[y[1:] for y in Y] if Y is not None else None, 
-                                                 scale_range=scale_range, xy=xy,
-                                                 do_flip=do_flip, rescale=rescale, unet=unet)
-
-
-# I have the omni flag here just in case, but it actually does not affect the tests
-def normalize_field(mu,omni=False):
-    if omni and OMNI_INSTALLED:
-        mu = omnipose.utils.normalize_field(mu) 
-    else:
-        mu /= (1e-20 + (mu**2).sum(axis=0)**0.5)
+def normalize_field(mu):
+    mu /= (1e-20 + (mu**2).sum(axis=0)**0.5)
     return mu
 
-
 def _X2zoom(img, X2=1):
     """ zoom in image
 
@@ -737,8 +657,8 @@ def _image_resizer(img, resize=512, to_uint8=False):
     return img
 
 
-def original_random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), 
-                             do_flip=True, rescale=None, unet=False):
+def random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), 
+                             do_flip=True, rescale=None, unet=False, random_per_image=True):
     """ augmentation by random rotation and resizing
         X and Y are lists or arrays of length nimg, with dims channels x Ly x Lx (channels optional)
         Parameters
@@ -760,6 +680,8 @@ def original_random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224),
         rescale: array, float (optional, default None)
             how much to resize images by before performing augmentations
         unet: bool (optional, default False)
+        random_per_image: bool (optional, default True)
+            different random rotate and resize per image
         Returns
         -------
         imgi: ND-array, float
@@ -785,27 +707,29 @@ def original_random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224),
             nt = 1
         lbl = np.zeros((nimg, nt, xy[0], xy[1]), np.float32)
 
-    scale = np.zeros(nimg, np.float32)
+    scale = np.ones(nimg, np.float32)
+    
     for n in range(nimg):
         Ly, Lx = X[n].shape[-2:]
 
-        # generate random augmentation parameters
-        flip = np.random.rand()>.5
-        theta = np.random.rand() * np.pi * 2
-        scale[n] = (1-scale_range/2) + scale_range * np.random.rand()
-        if rescale is not None:
-            scale[n] *= 1. / rescale[n]
-        dxy = np.maximum(0, np.array([Lx*scale[n]-xy[1],Ly*scale[n]-xy[0]]))
-        dxy = (np.random.rand(2,) - .5) * dxy
+        if random_per_image or n==0:
+            # generate random augmentation parameters
+            flip = np.random.rand()>.5
+            theta = np.random.rand() * np.pi * 2
+            scale[n] = (1-scale_range/2) + scale_range * np.random.rand()
+            if rescale is not None:
+                scale[n] *= 1. / rescale[n]
+            dxy = np.maximum(0, np.array([Lx*scale[n]-xy[1],Ly*scale[n]-xy[0]]))
+            dxy = (np.random.rand(2,) - .5) * dxy
 
-        # create affine transform
-        cc = np.array([Lx/2, Ly/2])
-        cc1 = cc - np.array([Lx-xy[1], Ly-xy[0]])/2 + dxy
-        pts1 = np.float32([cc,cc + np.array([1,0]), cc + np.array([0,1])])
-        pts2 = np.float32([cc1,
-                cc1 + scale[n]*np.array([np.cos(theta), np.sin(theta)]),
-                cc1 + scale[n]*np.array([np.cos(np.pi/2+theta), np.sin(np.pi/2+theta)])])
-        M = cv2.getAffineTransform(pts1,pts2)
+            # create affine transform
+            cc = np.array([Lx/2, Ly/2])
+            cc1 = cc - np.array([Lx-xy[1], Ly-xy[0]])/2 + dxy
+            pts1 = np.float32([cc,cc + np.array([1,0]), cc + np.array([0,1])])
+            pts2 = np.float32([cc1,
+                    cc1 + scale[n]*np.array([np.cos(theta), np.sin(theta)]),
+                    cc1 + scale[n]*np.array([np.cos(np.pi/2+theta), np.sin(np.pi/2+theta)])])
+            M = cv2.getAffineTransform(pts1,pts2)
 
         img = X[n].copy()
         if Y is not None:
